import {FC, useCallback, useEffect, useState} from "react";
import {Form, Input, Picker, View} from "@tarojs/components";
import Taro from "@tarojs/taro";
import {curriculum, Meeting, meetingAPi} from "@/api";
import {formatDate} from "@/utils/time";
import PopPut from "@/components/popPut/popPut";
import MyButton from "@/components/button/MyButton";
import styles from './form.module.scss'

import {Profile} from "@/store";
import DateTimePicker from "@/components/dateTimePicker/dateTimePicker";

const MeetingForm: FC = () => {
  const params = Taro.getCurrentInstance().router?.params as { id: string | undefined }
  const [manages, setManages] = useState<Manage[]>([])
  const [start, setStart] = useState<string>(formatDate(new Date(new Date().getTime() + 24 * 60 * 60 * 1000), "YY-MM-dd 08:00"))
  const [end, setEnd] = useState<string>(formatDate(new Date(new Date().getTime() + 24 * 60 * 60 * 1000), "YY-MM-dd 18:00"))
  const [depid, setDepid] = useState<number | null>(null)
  const [imgUrl, setImgUrl] = useState('')
  const [name, setName] = useState('')
  const [loading, setLoading] = useState(false)
  const [status, setStatus] = useState(0)  // 状态
  const [id, setId] = useState<number | undefined>(params.id ? Number(params.id) : undefined)
  const {company} = Profile.useContainer()

  function setData(res: Meeting | null) {
    /**
     * params.id 查看详情请求
     * 当onShow的时候 id === res.id 数据相同不进行以下流程
     */
    if (params.id || id !== res?.id) {
      setImgUrl('')
      setDepid(res?.dep_id || null)
      setName(res?.name || '')
      setStatus(res?.status || 0)
      setId(res?.id || 0)

      if (res) {
        setEnd(formatDate(new Date(res.estimate_end_time), "YY-MM-dd hh:mm"))
        setStart(formatDate(new Date(res.estimate_start_time), "YY-MM-dd hh:mm"))
      }
    }
    setLoading(false)
  }

  useEffect(() => {
    curriculum.department().then(res => {
      setManages(res.data)
    })
  }, [])

  Taro.useDidShow(() => {
    if (params.id != undefined) {
      meetingAPi.setMeeting(params.id).then(setData)
    }
  })

  const depChange = useCallback((e) => {
    setDepid(manages[Number(e.detail.value)]?.id)
  }, [manages])

  const change_start = useCallback((time: string) => {
    const mil = new Date(time).getTime()
    const endMil = new Date(end).getTime()
    if (mil > endMil) {
      Taro.showToast({title: '大于结束时间', icon: 'error'})
    } else {
      setStart(time)
    }
  }, [end])

  const change_end = useCallback((time: string) => {
    const mil = new Date(time).getTime()
    const startMil = new Date(start).getTime()
    if (mil < startMil) {
      Taro.showToast({title: '小于开始时间', icon: 'error'})
    } else {
      setEnd(time)
    }
  }, [start])





  const submit = useCallback(async (e) => {
    if (loading && status === 2) return;



    const values: Meeting = e.detail.value
    if (!values.name) {
      Taro.showToast({title: '请填写会议名称', icon: 'error'})
      return
    }

    setLoading(true)
    try {
      let req = meetingAPi.setMeetings
      if(!params.id){
        req = meetingAPi.addMeetings
      }
      await req({
        ...values,
        estimate_end_time: new Date(end).getTime(),
        estimate_start_time: new Date(start).getTime(),
        dep_id: depid,
        company_id: company?.id!,
        id: parseInt(params.id!),
      })
      setLoading(false)
      Taro.eventCenter.trigger('updateMeetingList')
      Taro.showToast({title: '操作成功', icon: 'success',mask:true})
      Taro.navigateBack()
    } catch (e) {
      setLoading(false)
    }
  }, [imgUrl, depid, end, start])









  return (
    <View className={styles.page}>

      <View className={styles.box}>




        <View className={styles.choice}>
          <Form onSubmit={submit}>
            <PopPut
              title='会议名称'
              chevron
              content={<Input
                className='input'
                name='name'
                value={name}
                disabled={status > 0}
                maxlength={11}
                placeholder='请输入会议标题'
                onInput={(e) => setName(e.detail.value)}
              />}
            />

            <DateTimePicker
              name='estimate_start_time'
              defaultValue={start}
              disabled={status === 1}
              onChange={(time) => change_start(time)}
            >
              <PopPut title='开始时间' content={start}/>
            </DateTimePicker>


            <DateTimePicker
              name='estimate_end_time'
              defaultValue={end}
              disabled={status === 1}
              onChange={(time) => change_end(time)}
            >
              <PopPut title='结束时间' content={end}/>
            </DateTimePicker>

            <Picker
              mode='selector'
              range={manages}
              onChange={depChange}
              disabled={status === 1}
              rangeKey='name'
              name='dep_id'>
              <PopPut title='选择部门' content={manages?.find(x => x.id == depid)?.name}/>
            </Picker>

            <MyButton
              formType='submit'
              fillet
              width={200}
              className={styles.button}
              loading={loading}>
              {params.id ? '保存':'新建'}
            </MyButton>
          </Form>
        </View>


      </View>
    </View>
  )
}

export default MeetingForm
