import {FC, useCallback, useState} from "react";
import {Image, View, Text} from "@tarojs/components";
import Taro, {getSetting, authorize} from "@tarojs/taro";
import { Meeting, meetingAPi} from "@/api";
import {formatDate} from "@/utils/time";
import MyButton from "@/components/button/MyButton";
import styles from './spotMeeting.module.scss'
import Icon from "@/components/icon";
import IconFont from "@/components/IconFont";
import CodeImg from "@/static/img/code.png"
import Spin from "@/components/spinner";
const SpotMeeting: FC = () => {
  const path = encodeURIComponent("/pages/meeting/meeting")
  const params = Taro.getCurrentInstance().router?.params as { id: string | undefined }
  const [start, setStart] = useState<string>(formatDate(new Date(new Date().getTime() + 24 * 60 * 60 * 1000), "MM-dd 08:00"))
  const [end, setEnd] = useState<string>(formatDate(new Date(new Date().getTime() + 24 * 60 * 60 * 1000), "MM-dd 18:00"))
  const [imgUrl, setImgUrl] = useState(CodeImg)
  const [name, setName] = useState('')
  const [status, setStatus] = useState(0)  // 状态
  const [id, setId] = useState<number | undefined>(params.id ? Number(params.id) : undefined)
  const pathUrl = process.env.TARO_ENV === 'h5' ? '/official/qrcode' : '/wechat/link'
  const [downStatus, setDownStatus] = useState<1|2|3>(1)

  function setData(res: Meeting | null) {
    /**
     * params.id 查看详情请求
     * 当onShow的时候 id === res.id 数据相同不进行以下流程
     */
    if (params.id || id !== res?.id) {
      setImgUrl('')
      setName(res?.name || '')
      setStatus(res?.status || 0)
      setId(res?.id || 0)

      if (res) {
        downUrl(res.id)
        setEnd(formatDate(new Date(res.estimate_end_time), "MM-dd hh:mm"))
        setStart(formatDate(new Date(res.estimate_start_time), "MM-dd hh:mm"))
      }
    }
  }

  Taro.useLoad(() => {
    if (params.id != undefined) {
      meetingAPi.setMeeting(params.id).then(setData)
    }

    Taro.eventCenter.on('UPDATE', () => {
      meetingAPi.setMeeting(params.id!).then(setData)
    })
  })

  Taro.useUnload(() => {
    Taro.eventCenter.off('UPDATE')
  })

  /** 下载二维码 */
  const handleWriteFile = useCallback(() => {
    console.log({imgUrl},'imgUrl')
    if (imgUrl == null) {
      Taro.showToast({title: '下载失败', icon: 'error'})
      return
    }
    Taro.saveImageToPhotosAlbum({
      filePath: imgUrl,
      success() {
        Taro.showModal({title: '下载成功'})
      },
      fail(err) {
        console.log(err,'err')
        Taro.showToast({title: '下载失败', icon: 'error'})
      }
    })
  }, [imgUrl])

  /** 查看保存权限 */
  const handleSaveCode = useCallback(() => {
    if( status === 2) {
      Taro.showToast({title: '会议已结束，无法下载', icon: 'none'})
      return
    }
    if (process.env.TARO_ENV !== 'h5') {
      getSetting({
        success: function ({authSetting}) {
          //没有权限则申请
          if (!authSetting["scope.writePhotosAlbum"]) {
            authorize({
              scope: "scope.writePhotosAlbum",
              success: () => {
                //存储二维码
                handleWriteFile()
              },
              fail() {
                Taro.showToast({title: '保存授权失败', icon: "error"})
              }
            });
          } else {
            handleWriteFile()
          }
        },
      });
    } else {
      Taro.showToast({title: '请截图保存', icon: 'none'})
    }
  }, [imgUrl]);

  /** 获取二维码地址 */
  const downUrl = (id: number,) => {
    if(status === 2){
      setDownStatus(2)
      return
    }
    // Taro.showLoading({title: '加载二维码'})
    const url = process.env.TARO_APP_API + pathUrl + '?meeting_id=' + id + '&path=' + path

    let fileName = new Date().valueOf();
    let filePath = Taro.env.USER_DATA_PATH + '/' + fileName + '.jpg'
    console.log(filePath,'filePath')


    if (process.env.TARO_ENV !== 'h5') {
      Taro.downloadFile({
        url,
        filePath:filePath,
        success(res) {
          console.log(res,'res')
          setImgUrl(res.filePath)
          setDownStatus(3)

        },
        fail() {
          setDownStatus(2)
          // Taro.hideLoading()
          // Taro.showModal({title: '二维码生成失败'})
        }
      })
    } else {
      setTimeout(() => {
        setImgUrl(url)
      })

    }
  }





  /** 开始 */
  async function onStart() {
    try {
      await meetingAPi.start(id!)
      setStatus(1)
      Taro.showModal({
        title: '现场会开始',
        content: '扫描二维码可加入'
      })
    } catch (e) {
    }
  }

  async function onEnd() {
    try {
      await meetingAPi.end(id!)
      setImgUrl('')
      setStatus(2)
    } catch (e) {
    }
  }

  async function onDel(){
    Taro.showModal({
      title:'友情提示',
      content:'是否确认删除',
      success: async (res) => {
        if(res.confirm){
          try {
            Taro.showLoading()
            await meetingAPi.del(id!)
            Taro.showToast({title: '删除成功', icon: 'none',mask: true})
            Taro.eventCenter.trigger('deleteMeeting', id)
            Taro.navigateBack()
          }catch (err){
            Taro.showToast({title: '删除失败，稍后重试', icon: 'none',mask: true})
          } finally {
            Taro.hideLoading()
          }
        }
      }
    })

  }

  return (
    <View className={styles.page}>

      <View className={styles.box}>
        <View className="flex justify-between px-3">
          <Text className="font-36 bold">{name}</Text>
          {
            status === 0 &&
            <Icon name={'edit'} onClick={()=>{
              Taro.navigateTo({
                url:`/pages/manage/meetings/form/form?id=${id}`
              })
            }} />
          }
        </View>
        {process.env.TARO_ENV === 'h5' && <View className='text-center text-muted'>微信公众号，浏览器中请截图保存</View>}

        <View className="hr-solid mt-3 mx-3"></View>
        <View className={styles.code}>
          {/*<Img src={imgUrl}  width='200px' height='200px'></Img>*/}
              <Image
                onLongPress={handleSaveCode}
                src={imgUrl}
                mode='aspectFit'
                style={{width: '200px'}}
                fadeIn/>
          {
            downStatus < 3 &&
            <View  className="absolute top left bottom right flex flex-column justify-center align-center">
              {downStatus === 1 && <Spin enable />}
              {downStatus === 2 && <IconFont name={'exclamation'} shape={'triangle'} color={'#f00'} fill size={30} />}
            </View>
          }
            {/*:*/}
            {/*<View className="flex justify-center align-center"*/}
            {/*  style={{width:'200px',height:'240px'}}*/}
            {/*>*/}
            {/*  <AtLoadMore*/}
            {/*    status={'loading'}*/}
            {/*    style={{fontSize:'30rpx'}}*/}
            {/*  />*/}
            {/*</View>*/}
          {/*}*/}
        </View>
        <View className="font-30 py-3">
          <Text>{start}</Text>
          <Text className="mx-1">至</Text>
          <Text>{end}</Text>
        </View>
        <View className={styles.tips}>
          {/*<Icon name={'alert-circle'} color={'#45D4A8'}></Icon>*/}
          <IconFont name={'info'} shape={'circle'} fill color={'#45D4A8'} size={14}></IconFont>
          <Text className="ml-1 text-success">长按保存二维码到相册</Text>
        </View>
        <View className="mt-4">
          {
            status === 0 && <MyButton
              fillet
              width={300}
              className={styles.button}
              onClick={onStart}>
              开始
            </MyButton>
          }
          {status === 1 && <MyButton fillet type='warn' width={300} className={styles.buttonEnd} onClick={onEnd}>结束</MyButton>}
        </View>

      </View>
      {status === 0 && <MyButton type={"default"} fillet width={320} className={styles.buttonDel} onClick={onDel}>删除</MyButton>}
    </View>
  )
}

export default SpotMeeting
